#!/usr/bin/env python

import sys
import gzip

#####################################################
#This is coincident level 0!!
#Order of scintillators is irrelevent!
#####################################################

files = sys.argv[1:]

BIT0_4 = 31
BIT5 = 1 << 5
BIT7 = 1 << 7

# For DAQ status
BIT0 = 1 # 1 PPS interrupt pending
BIT1 = 1 << 1 # Trigger interrupt pending
BIT2 = 1 << 2 # GPS data possible corrupted
BIT3 = 1 << 3 # Current or last 1PPS rate not within range

def time_to_seconds(time, correction):
    '''
    Convert hhmmss,xxx string int seconds since day start
    '''
#   print time,correction
    tfields = time.split(".")
    t = tfields[0]
    secs_since_day_start = int(t[0:2])*3600+int(t[2:4])*60+int(t[4:6])
    evt_time = secs_since_day_start + int(tfields[1])/1000.0+int(correction)/1000.0
    return round(evt_time)
    #filename = sys.argv[1]
    f = open(sys.argv[1])
    

for filename in files:
    ch0 = float(sys.argv[2]) #0 or 1
    ch1 = float(sys.argv[3]) #0 or 1
    ch2 = float(sys.argv[4]) #0 or 1
    ch3 = float(sys.argv[5]) #0 or 1
    dpch0 = float(sys.argv[6]) #0 or 1
    dpch1 = float(sys.argv[7]) #0 or 1
    dpch2 = float(sys.argv[8]) #0 or 1
    dpch3 = float(sys.argv[9]) #0 or 1
    
    
    muon = {0:False,1:False,2:False,"Time":0.}
    freq = 25e6 # 25 MHz
    last_onepps = 0
    last_muon = 0
    wait_fe0 = False
    time_ch0 = 0.
    wait_fe1 = False
    time_ch1 = 0.
    wait_fe2 = False
    time_ch2 = 0.
    wait_fe3 = False
    time_ch3 = 0.

    muon_start = 0.

    nmuons = 0
    last_pulse = 0.
    decay_start_time_ch0 = 0.
    decay_waiting_ch0 = False
    decay_start_time_ch1 = 0.
    decay_waiting_ch1 = False
    decay_start_time_ch2 = 0.
    decay_waiting_ch2 = False
    decay_start_time_ch3 = 0.
    decay_waiting_ch3 = False
    
    
    last_seconds = 0.
    last_time = 0.
    last_triggercount = 0
    switched_onepps = False
    last_onepps = 0
    onepps_count = 0
    counter = 0.0
   

    if filename.endswith('.gz'):
        f = gzip.open(filename)
    else:
        f = open(sys.argv[1])
    for line in f:
        
        fields = line.rstrip("\n").split(" ")
        #print "fields: ",fields
        #print "line: ",line
        # Ignore malformed lines
        if len(fields) != 16:
            continue
        #Ignore everything that is not trigger data
        if len(fields[0]) != 8:
            continue

        try:
            int(fields[len(fields)-1])
        except ValueError:
            continue
 
        trigger_count = int(fields[0],16)
        onepps_count = int(fields[9],16)
    
        re_0 = (fields[1] >= "00")
        re_1 = (fields[3] >= "00")
        re_2 = (fields[5] >= "00")
        re_3 = (fields[7] >= "00")
        fe_0 = (fields[2] >= "00")
        fe_1 = (fields[4] >= "00")
        fe_2 = (fields[6] >= "00")
        fe_3 = (fields[8] >= "00")

        #check the single pulse for channel

        if (ch0==1 and fields[1] == "00"):
            continue
        if (ch1==1 and fields[3] == "00"):
            continue
        if (ch2==1 and fields[5] == "00"):
            continue
        if (ch3==1 and fields[7] == "00"):
            continue
        if (ch0==1 and ch1==1 and (fields[1] == "00" or fields[3] == "00")):
            continue
        if (ch0==1 and ch2==1 and (fields[1] == "00" or fields[5] == "00")):
            continue
        if (ch0==1 and ch3==1 and (fields[1] == "00" or fields[7] == "00")):
            continue
        if (ch1==1 and ch2==1 and (fields[3] == "00" or fields[5] == "00")):
            continue
        if (ch1==1 and ch3==1 and (fields[3] == "00" or fields[7] == "00")):
            continue
        if (ch2==1 and ch3==1 and (fields[5] == "00" or fields[7] == "00")):
            continue
        if (ch0==1 and ch1==1 and ch2==1 and (fields[1] == "00" and fields[3] == "00" or fields[5] == "00")):
            continue
        if (ch1==1 and ch2==1 and ch3==1 and (fields[3] == "00" and fields[5] == "00" or fields[7] == "00")):
            continue
        
        #check for double pulse

        if fields[1] >= "80":
            for line in f:
                fields_next = line.rstrip("\n").split(" ")
                #define the different options
                
                if 0==0: 
                #current analysis for one event. Note, that one event can be described by more than one line!
                    if last_onepps != onepps_count:
                        if onepps_count > last_onepps:
                            freq  = float(onepps_count - last_onepps)
                        else:
                            freq = float(0xFFFFFFFF + onepps_count - last_onepps)
           
                        prevlast_onepps = last_onepps
                        last_onepps = onepps_count
                        switched_onepps = True

                    time = fields[10]
                    correction = fields[15]
                    seconds0 = time_to_seconds(time,correction)+(trigger_count - onepps_count)/freq
                    seconds1 = time_to_seconds(time,correction)+(trigger_count - onepps_count)/freq
                    seconds2 = time_to_seconds(time,correction)+(trigger_count - onepps_count)/freq
                    seconds3 = time_to_seconds(time,correction)+(trigger_count - onepps_count)/freq
                             
                    if time == last_time and switched_onepps:
                        print "Correcting delayed onepps switch:",seconds0,line
                        seconds0 = time_to_seconds(time,correction)+(trigger_count - prevlast_onepps)/freq
                        seconds1 = time_to_seconds(time,correction)+(trigger_count - prevlast_onepps)/freq
                        seconds2 = time_to_seconds(time,correction)+(trigger_count - prevlast_onepps)/freq
                        seconds3 = time_to_seconds(time,correction)+(trigger_count - prevlast_onepps)/freq
                    else:
                        last_time = time
                        switched_onepps = False
        
                    if trigger_count < last_triggercount and not switched_onepps:
                        print "Correcting trigger count rollover:",seconds0,line
                        seconds0 += int(0xFFFFFFFF)/freq
                        seconds1 += int(0xFFFFFFFF)/freq
                        seconds2 += int(0xFFFFFFFF)/freq
                        seconds3 += int(0xFFFFFFFF)/freq

                    else:
                        last_triggercount = trigger_count
        
                    #if last_seconds > seconds0:
                     #   print "Wrong event order",seconds0,line
                      #  continue
		
                    last_seconds = seconds0
                    print "seconds0:",seconds0
                    print "decay_start_time_ch0: ",decay_start_time_ch0
                    print "decay_start_time_ch1: ",decay_start_time_ch1
                    print "decay_start_time_ch2: ",decay_start_time_ch2
                    print "decay_start_time_ch3: ",decay_start_time_ch3
                    print "difference: ",seconds0-decay_start_time_ch0
                    
                    pulse_ch0 = False
                    pulse_ch1 = False
                    pulse_ch2 = False
                    pulse_ch3 = False

                    #single channel++++++++++++++++++++++++++++

                    if dpch0==1 and (seconds0 !=decay_start_time_ch1 ) and (seconds0 != decay_start_time_ch2) and (seconds0 != decay_start_time_ch3) :
                        print "dpch0"
                        print "Decay ch0 %10.8f microseconds"%(1e6*(seconds0 - decay_start_time_ch0),)
                        if decay_waiting_ch0:
                            if re_0:
                                wait_fe0 = True
                                time_ch0 = seconds0
                            if time_ch0 - seconds0 > 50e-9:
                                wait_f0 = False
                                decay_waiting_ch0 = False
                            if fe_0 and wait_fe0:
                                print "Pulse ch0",seconds0,line
                                pulse_ch0 = True
                                wait_fe0 = False
                            if decay_waiting_ch0 and seconds0 - decay_start_time_ch0 > 20:
                                print "No decay",seconds0,decay_start_time_ch0, seconds0 - decay_start_time_ch0
                                decay_waiting_ch0 = False
                            else:
                                decay_waiting_ch0 = False   
                                print "Decay ch0 %10.8f ch0microseconds"%((seconds0 - decay_start_time_ch0),)
                        else:
                            decay_waiting_ch0 = False
                    else: 
                        print "no decay in channel 0"
                        
                    if dpch1==1 and (seconds1 != decay_start_time_ch0) and (seconds1 != decay_start_time_ch2) and (seconds1 != decay_start_time_ch3) :
                        
                        if  decay_waiting_ch1:
                            if re_1:
                                wait_fe1 = True
                                time_ch1 = seconds1
                            if time_ch1 - seconds1 > 50e-9:
                                wait_f1 = False
                            if fe_1 and wait_fe1:
                                print "Pulse ch1",seconds1,line
                                pulse_ch1 = True
                                wait_fe1 = False
                            if decay_waiting_ch1 and seconds1 - decay_start_time_ch1 > 20:
                                print "No decay",seconds1,decay_start_time_ch1,seconds1 - decay_start_time_ch1
                                decay_waiting_ch1 = False
                            else:
                                decay_waiting_ch1 = False           
                                print "Decay ch1 %10.8f ch1microseconds"%((seconds1 - decay_start_time_ch1),)
                                
                        else:
                            decay_waiting_ch1 = False    
                            
                    else: 
                        print "no decay in channel 1"
                
                    if dpch2==1 and (seconds2 != decay_start_time_ch0) and (seconds2 != decay_start_time_ch1) and (seconds2 != decay_start_time_ch3):
                        if  decay_waiting_ch2:
                            if re_2:
                                wait_fe2 = True
                                time_ch2 = seconds2
                            if time_ch2 - seconds2 > 50e-9:
                                wait_f2 = False
                                decay_waiting_ch2 = False
                            if fe_2 and wait_fe2:
                                print "Pulse ch2",seconds2,line
                                pulse_ch2 = True
                                wait_fe2 = False
                            if decay_waiting_ch2 and seconds2 - decay_start_time_ch2 > 20:
                                print "No decay",seconds2,decay_start_time_ch2, seconds2 - decay_start_time_ch2
                                decay_waiting_ch2 = False
                            else:
                                decay_waiting_ch2 = False           
                                print "Decay ch2 %10.8f ch2microseconds"%((seconds2 - decay_start_time_ch2),)
                    
                        else:
                            decay_waiting_ch2 = False
        
                    else: 
                        print "no decay in channel 2"

                    if dpch3==1 and (seconds3 != decay_start_time_ch0) and (seconds3 != decay_start_time_ch1) and (seconds3 != decay_start_time_ch2) :     
                        if  decay_waiting_ch3:
                            if re_3:
                                wait_fe3 = True
                                seconds3 = time_to_seconds(time,correction)+(trigger_count - onepps_count)/freq
                                time_ch3 = seconds3
                            if time_ch3 - seconds3 > 50e-9:
                                wait_f3 = False
                            if fe_3 and wait_fe3:
                                print "Pulse ch3",seconds3,line
                                pulse_ch3 = True
                                wait_fe3 = False
                            if decay_waiting_ch3 and seconds3 - decay_start_time_ch3 > 20:
                                print "No decay",seconds3,decay_start_time_ch3,seconds3 - decay_start_time_ch3
                                decay_waiting_ch3 = False
                            else:
                                decay_waiting_ch3 = False            
                                print "Decay ch3 %10.8f ch3microseconds"%((seconds3 - decay_start_time_ch3),)
                        else:
                            decay_waiting_ch3 = False
                    else: 
                        print "no decay in channel 3"
                
                    if re_0 and dpch0==1:
                        wait_fe0 = True
                        time_ch0 = seconds0
                        if time_ch0 - seconds0 > 50e-9:
                            wait_f0 = False
                        if fe_0 and wait_fe0:
                            print "Pulse ch0",seconds0,line
                            decay_start_time_ch0 = 0
                            decay_start_time_ch0 = seconds0
                            decay_waiting_ch0 = True
                            pulse_ch0 = True
                            wait_fe0 = False
                        else:
                            decay_waiting_ch0 = False
                             
                    if re_1 and dpch1==1:
                        wait_fe1 = True
                        time_ch1 = seconds1
                        if time_ch1 - seconds1 > 50e-9:
                            wait_f1 = False
                        if fe_1 and wait_fe1:
                            #print "Pulse ch1",seconds1,line
                            decay_start_time_ch1 = 0
                            decay_start_time_ch1 = seconds1
                            decay_waiting_ch1 = True
                            pulse_ch1 = True
                            wait_fe1 = False
                        else:
                            decay_waiting_ch1 = False
                
                            
                    if re_2 and dpch2==1:
                        wait_fe2 = True
                        time_ch2 = seconds2
                        if time_ch2 - seconds2 > 50e-9:
                            wait_f2 = False
                        if fe_2 and wait_fe2:
                            print "Pulse ch2",seconds2,line
                            decay_start_time_ch2 = seconds2
                            decay_waiting_ch2 = True
                            pulse_ch2 = True
                            wait_fe2 = False
                        else:
                            decay_waiting_ch2 = False
                            
                            
                    if re_3 and dpch3==1:
                        wait_fe3 = True
                        time_ch3 = seconds3
                        if time_ch3 - seconds3 > 50e-9:
                            wait_f3 = False
                        if fe_3 and wait_fe3:
                            print "Pulse ch2",seconds3,line
                            decay_start_time_ch3 = seconds3
                            decay_waiting_ch3 = True
                            pulse_ch3 = True
                            wait_fe3 = False
                        else:
                            decay_waiting_ch3 = False
                            
                        #seconds0=decay_start_time_ch0 
                        #seconds1=decay_start_time_ch1
                        #seconds2=decay_start_time_ch2
                        #seconds3=decay_start_time_ch3

                if fields_next[1]>= "80":
                    previous_fe_0 = fe_0
                    previous_re_1 = re_1
                    previous_fe_1 = fe_1
                    previous_re_2 = re_2
                    previous_fe_2 = fe_2
                    previous_re_3 = re_3
                    previous_fe_3 = fe_3
                    break
            



"""
Provides a simple DAQ card simulation, so that software can be tested
"""

import time
import Queue
import numpy as n
import os

try:
	import zmq
except ImportError:
	print "no zmq installed..."
	
from random import choice
	
class SimDaq():

    def __init__(self, logger,usefile="simdaq.txt",createfakerates=True):
        
        # TODO:
        # Modify this in such a way that
        # an arbitrary file can be read and computed...
        
        self.logger = logger
        self.ini = True

        self._pushed_lines = 0
        self._lines_to_push = 10
        self._simdaq_file = os.path.split(os.path.abspath(__file__))[0] + os.sep + usefile
        self._daq = open(self._simdaq_file)
        self._inWaiting = True 
        self._return_info = False
        
        #self._fakerates = createfakerates
        self._scalars_ch0 = 0
        self._scalars_ch1 = 0
        self._scalars_ch2 = 0
        self._scalars_ch3 = 0
        self._scalars_trigger = 0
        self._scalars_to_return = ''

    def _physics(self):
        """
        This routine will increase the scalars variables using predefined rates
        Rates are drawn from Poisson distributions
        """

        def format_to_8digits(hexstring):
            return hexstring.zfill(8)
        
        
        # draw rates from a poisson distribution,
        self._scalars_ch0 += int(choice(n.random.poisson(12,100)))
        self._scalars_ch1 += int(choice(n.random.poisson(10,100)))
        self._scalars_ch2 += int(choice(n.random.poisson(8,100)))
        self._scalars_ch3 += int(choice(n.random.poisson(11,100)))
        self._scalars_trigger += int(choice(n.random.poisson(2,100)))
        self._scalars_to_return = 'DS S0=' + format_to_8digits(hex(self._scalars_ch0)[2:]) + ' S1=' + format_to_8digits(hex(self._scalars_ch1)[2:]) + ' S2=' + format_to_8digits(hex(self._scalars_ch2)[2:]) + ' S3=' + format_to_8digits(hex(self._scalars_ch3)[2:]) + ' S4=' + format_to_8digits(hex(self._scalars_trigger)[2:])
        self.logger.debug("Scalars to return %s" %self._scalars_to_return)


    def readline(self):
        """
        read dummy pulses from the simdaq file till
        the configured value is reached
        """
        if self.ini:
            self.ini = False
            return "T0=42  T1=42  T2=42  T3=42"

        if self._return_info:
            self._return_info = False
            return self._scalars_to_return

        self._pushed_lines += 1
        if self._pushed_lines < self._lines_to_push:
            line = self._daq.readline()
            if not line:
                self._daq = open(self._simdaq_file)
                self.logger.debug("File reloaded")
                line = self._daq.readline()

            return line
        else:
            self._pushed_lines = 0
            self._inWaiting = False
            return self._daq.readline()
            

    def write(self,command):
        """
        Trigger a simulated daq response with command
        """
        self.logger.debug("got the following command %s" %command.__repr__())
        if "DS" in command:
            self._return_info = True


    def inWaiting(self):
        """
        simulate a busy DAQ
        """
        if self._inWaiting:
            time.sleep(0.1)
            self._physics()
            return True

        else:
            self._inWaiting = True
            return False
        
class SimDaqConnection(object):

    def __init__(self, inqueue, outqueue, logger):

        self.logger = logger
        self.port = SimDaq(self.logger)
        self.inqueue = inqueue
        self.outqueue = outqueue
        self.running = 1

    def read(self):
        """
        Simulate DAQ I/O
        """
        while self.running:
            try:
                self.logger.debug("inqueue size is %d" %self.inqueue.qsize())
                while self.inqueue.qsize():
                    try:
                        #print self.inqueue.get(0)
                        self.port.write(str(self.inqueue.get(0))+"\r")
                    except Queue.Empty:
                        self.logger.debug("Queue empty!")
            except NotImplementedError:
                self.logger.debug("Running Mac version of muonic.")
                while True:
                    try:
                        self.port.write(str(self.inqueue.get(timeout=0.01))+"\r")
                    except Queue.Empty:
                        pass

            while self.port.inWaiting():
                self.outqueue.put(self.port.readline().strip())
            time.sleep(0.02)

class SimDaqServer(object):

    def __init__(self, port, logger):

        self.logger = logger
        self.port = SimDaq(self.logger)
        self.running = 1
        self.setup_socket(port)
        

    def setup_socket(self,port,adress="127.0.0.1"):
        #port = "5556"
        context = zmq.Context()
        self.socket = context.socket(zmq.PAIR)
        self.socket.bind("tcp://%s:%s" %(adress, port))
        self.socket_adress = adress
        self.socket_port = port

    def serve(self):
        while True:
            self.read()
		
    def read(self):
        """
        Simulate DAQ I/O
        """
        while self.running:
            
            #self.logger.debug("inqueue size is %d" %self.inqueue.qsize())
            msg = self.socket.recv_string()
            print msg
            try:
                    #print self.inqueue.get(0)
                self.port.write(str(msg)+"\r")
            except Queue.Empty:
                self.logger.debug("Queue empty!")
            
            while self.port.inWaiting():
                self.socket.send_string(self.port.readline().strip())
            time.sleep(0.02)

if __name__ == "__main__":
    
    import logging
    logger = logging.getLogger()
    x = SimDaqServer("5556",logger)
    x.serve()



"""
Provides the main window for the gui part of muonic
"""

# Qt4 imports
from PyQt4 import QtGui
from PyQt4 import QtCore

# numpy
import numpy as n

# stdlib imports
import datetime

import os
import shutil
import time
import webbrowser


# muonic imports
from ..analysis import PulseAnalyzer as pa
from ..daq.DAQProvider import DAQIOError
from ..__version__ import __version__,__source_location__,\
__docs_hosted_at__
from .styles import LargeScreenMPStyle

from .MuonicDialogs import ThresholdDialog,ConfigDialog,HelpDialog,DecayConfigDialog,PeriodicCallDialog,AdvancedDialog
from .MuonicPlotCanvases import ScalarsCanvas,LifetimeCanvas,PulseCanvas
from .MuonicWidgets import VelocityWidget,PulseanalyzerWidget,DecayWidget,DAQWidget,RateWidget, GPSWidget, StatusWidget

DOCPATH  = (os.path.dirname(os.path.abspath(__file__)) + os.sep + '..' + os.sep + 'docs' + os.sep + 'html')
# this is hard-coded! There must be a better solution...
# if you change here, you have to change in setup.py!
DATAPATH = os.getenv('HOME') + os.sep + 'muonic_data'

if not os.path.isdir(DATAPATH):
    os.mkdir(DATAPATH, 761)

#from TabWidget import TabWidget

tr = QtCore.QCoreApplication.translate

class MainWindow(QtGui.QMainWindow):
    """
    The main application
    """

    def __init__(self, daq, logger, opts,  win_parent = None):
        QtGui.QMainWindow.__init__(self, win_parent)
        self.daq = daq
        self.opts = opts
        self.DATAPATH = DATAPATH
        self.daq_msg = False # try to establish upstream compatibility
                             # B. introduced a DAQMessage here, but I think
                             # this is overkill
                             # however, making it a member allows to distribute
                             # it more easily to daughter widgets

        # we have to ensure that the DAQcard does not sent
        # any automatic status reports every x seconds
        #FIXME: there was an option for that!
        self.nostatus = opts.nostatus
        if self.nostatus:
            self.daq.put('ST 0')

        self.setWindowTitle(QtCore.QString("muonic") )
        self.statusbar = QtGui.QMainWindow.statusBar(self)
        self.logger  = logger
        desktop = QtGui.QDesktopWidget()
        screen_size = QtCore.QRectF(desktop.screenGeometry(desktop.primaryScreen()))
        screen_x = screen_size.x() + screen_size.width()
        screen_y = screen_size.y() + screen_size.height()
        self.logger.info("Screen with size %i x %i detected!" %(screen_x,screen_y))

        # FIXME: make it configurable
        # now simply set to 1600 x 1200
        if screen_x * screen_y >= 1920000:
            LargeScreenMPStyle()
        self.date = time.gmtime()
        # put the file in the data directory
        # we chose a global format for naming the files -> decided on 18/01/2012
        # we use GMT times
        # " Zur einheitlichen Datenbezeichnung schlage ich folgendes Format vor:
        # JJJJ-MM-TT_y_x_vn.dateiformat (JJJJ-Jahr; MM-Monat; TT-Speichertag bzw.
        # Beendigung der Messung; y: G oder L ausw?hlen, G steht f?r **We add R for rate P for pulses and RW for RAW **
        # Geschwindigkeitsmessung/L f?r Lebensdauermessung; x-Messzeit in Stunden;
        # v-erster Buchstabe Vorname; n-erster Buchstabe Familienname)."
        # TODO: consistancy....        
 
        # the time when the rate measurement is started
        self.now = datetime.datetime.now()
        self.filename = os.path.join(DATAPATH,"%s_%s_HOURS_%s%s" %(self.now.strftime('%Y-%m-%d_%H-%M-%S'),"R",opts.user[0],opts.user[1]) )
        self.rawfilename = os.path.join(DATAPATH,"%s_%s_HOURS_%s%s" %(self.now.strftime('%Y-%m-%d_%H-%M-%S'),"RAW",opts.user[0],opts.user[1]) )
        self.raw_mes_start = False

        self.decayfilename = os.path.join(DATAPATH,"%s_%s_HOURS_%s%s" %(self.now.strftime('%Y-%m-%d_%H-%M-%S'),"L",opts.user[0],opts.user[1]) )
        self.pulse_mes_start = None
        self.writepulses = False
        if self.writepulses:
                self.daq.put('CE')
                self.pulsefilename = os.path.join(DATAPATH,"%s_%s_HOURS_%s%s" %(self.now.strftime('%Y-%m-%d_%H-%M-%S'),"P",opts.user[0],opts.user[1]) )
                self.pulse_mes_start = self.now
        else:
                self.pulsefilename = ''
                self.pulse_mes_start = False

        self.daq.put('TL') # get the thresholds
        time.sleep(0.5) #give the daq some time to ract
        self.threshold_ch0 = 300
        self.threshold_ch1 = 300
        self.threshold_ch2 = 300
        self.threshold_ch3 = 300

        self.channelcheckbox_0 = True
        self.channelcheckbox_1 = True
        self.channelcheckbox_2 = True
        self.channelcheckbox_3 = True
        self.coincidencecheckbox_0 = True
        self.coincidencecheckbox_1 = False
        self.coincidencecheckbox_2 = False
        self.coincidencecheckbox_3 = False
        self.vetocheckbox = False
        self.vetocheckbox_0 = False
        self.vetocheckbox_1 = False
        self.vetocheckbox_2 = False
        
        while self.daq.data_available():
            try:
                msg = self.daq.get(0)
                self.get_thresholds_from_queue(msg)

            except DAQIOError:
                self.logger.debug("Queue empty!")

        self.coincidence_time = 0.

        self.daq.put('DC') # get the channelconfig
        time.sleep(0.5) #give the daq some time to ract
        while self.daq.data_available():
            try:
                msg = self.daq.get(0)
                self.get_channels_from_queue(msg)

            except DAQIOError:
                self.logger.debug("Queue empty!")
                
        # the pulseextractor for direct analysis
        self.pulseextractor = pa.PulseExtractor(pulsefile=self.pulsefilename) 
        self.pulses = None

        # A timer to periodically call processIncoming and check what is in the queue
        self.timer = QtCore.QTimer()
        QtCore.QObject.connect(self.timer,
                           QtCore.SIGNAL("timeout()"),
                           self.processIncoming)
        
        #tab widget to hold the different physics widgets
        self.tabwidget = QtGui.QTabWidget(self)
        #pal = QtGui.QPalette()
        #pal.setColor(QtGui.QPalette.Window, QtGui.QColor(0,222,0))
        #self.tabwidget.setPalette(pal)
        #this is a stupid comment for getting upload permission ;)  
        self.tabwidget.mainwindow = self.parentWidget()

        self.timewindow = opts.timewindow #5.0
        self.logger.info("Timewindow is %4.2f" %self.timewindow)

        self.tabwidget.addTab(RateWidget(logger,parent = self),"Muon Rates")
        self.tabwidget.ratewidget = self.tabwidget.widget(0)

        self.tabwidget.addTab(PulseanalyzerWidget(logger,parent = self),"Pulse Analyzer")
        self.tabwidget.pulseanalyzerwidget = self.tabwidget.widget(1)

        self.tabwidget.addTab(DecayWidget(logger,parent = self),"Muon Decay")
        self.tabwidget.decaywidget = self.tabwidget.widget(2)
      
        self.tabwidget.addTab(VelocityWidget(logger, parent = self),"Muon Velocity")
        self.tabwidget.velocitywidget = self.tabwidget.widget(3)

        self.tabwidget.addTab(StatusWidget(logger,parent=self),"Status")
        self.tabwidget.statuswidget = self.tabwidget.widget(4)

        self.tabwidget.addTab(DAQWidget(logger,parent=self),"DAQ Output")
        self.tabwidget.daqwidget = self.tabwidget.widget(5)

        self.tabwidget.addTab(GPSWidget(logger,parent=self),"GPS Output")
        self.tabwidget.gpswidget = self.tabwidget.widget(6)

        # widgets which should be calculated in processIncoming.
        # The widget is only calculated when it is set to active (True)
        #  via widget.active
        # only widgets which need pulses go here
        self.tabwidget.pulse_widgets = [self.tabwidget.pulseanalyzerwidget,\
                                            self.tabwidget.velocitywidget,\
                                            self.tabwidget.decaywidget]
        # the others go in this list
        self.tabwidget.nopulse_widget = [self.tabwidget.ratewidget]

        # widgets which shuld be dynmacally updated by the timer should be in this list
        self.tabwidget.dynamic_widgets = [self.tabwidget.decaywidget,\
                                          self.tabwidget.pulseanalyzerwidget,\
                                          self.tabwidget.velocitywidget,\
                                          self.tabwidget.ratewidget]

        self.widgetupdater = QtCore.QTimer()
        QtCore.QObject.connect(self.widgetupdater,
                           QtCore.SIGNAL("timeout()"),
                           self.widgetUpdate)
 
        self.setCentralWidget(self.tabwidget)
        # provide buttons to exit the application
        exit = QtGui.QAction(QtGui.QIcon('/usr/share/icons/gnome/24x24/actions/exit.png'), 'Exit', self)
        exit.setShortcut('Ctrl+Q')
        exit.setStatusTip('Exit application')

        self.connect(exit, QtCore.SIGNAL('triggered()'), QtCore.SLOT('close()') )

        # prepare the config menu
        config = QtGui.QAction(QtGui.QIcon(''),'Channel Configuration', self)
        config.setStatusTip('Configure the Coincidences and channels')
        self.connect(config, QtCore.SIGNAL('triggered()'), self.config_menu)

        # prepare the advanced config menu
        advanced = QtGui.QAction(QtGui.QIcon(''),'Advanced Configurations', self)
        advanced.setStatusTip('Advanced configurations')
        self.connect(advanced, QtCore.SIGNAL('triggered()'), self.advanced_menu)       

        # prepare the threshold menu
        thresholds = QtGui.QAction(QtGui.QIcon(''),'Thresholds', self)
        thresholds.setStatusTip('Set trigger thresholds')
        self.connect(thresholds, QtCore.SIGNAL('triggered()'), self.threshold_menu)
               
        # helpmenu
        helpdaqcommands = QtGui.QAction(QtGui.QIcon('icons/blah.png'),'DAQ Commands', self)
        self.connect(helpdaqcommands, QtCore.SIGNAL('triggered()'), self.help_menu)

        # sphinx-documentation
        sphinxdocs = QtGui.QAction(QtGui.QIcon('icons/blah.png'), 'Technical documentation', self)
        self.connect(sphinxdocs,QtCore.SIGNAL('triggered()'),self.sphinxdoc_menu)
        
        # manual
        manualdocs = QtGui.QAction(QtGui.QIcon('icons/blah.png'), 'Manual', self)
        self.connect(manualdocs,QtCore.SIGNAL('triggered()'),self.manualdoc_menu)
   
        # about
        aboutmuonic = QtGui.QAction(QtGui.QIcon('icons/blah.png'),'About muonic', self)
        self.connect(aboutmuonic, QtCore.SIGNAL('triggered()'), self.about_menu)
        
        # create the menubar and fill it with the submenus
        menubar  = self.menuBar()
        filemenu = menubar.addMenu(tr('MainWindow','&File'))
        filemenu.addAction(exit)
        settings = menubar.addMenu(tr('MainWindow', '&Settings'))
        settings.addAction(config)
        settings.addAction(thresholds)
        settings.addAction(advanced)

        helpmenu = menubar.addMenu(tr('MainWindow','&Help'))
        helpmenu.addAction(helpdaqcommands)
        helpmenu.addAction(sphinxdocs)
        helpmenu.addAction(manualdocs)
        helpmenu.addAction(aboutmuonic)

        self.processIncoming()
        self.timer.start(1000)
        self.widgetupdater.start(self.timewindow*1000) 

    #the individual menus
    def threshold_menu(self):
        """
        Shows the threshold dialogue
        """
        # get the actual Thresholds...
        self.daq.put('TL')
        # wait explicitely till the thresholds get loaded
        self.logger.info("loading threshold information..")
        time.sleep(1.5)
        threshold_window = ThresholdDialog(self.threshold_ch0,self.threshold_ch1,self.threshold_ch2,self.threshold_ch3)
        rv = threshold_window.exec_()
        if rv == 1:
            commands = []
            for ch in ["0","1","2","3"]:
                val = threshold_window.findChild(QtGui.QSpinBox,QtCore.QString("thr_ch_" + ch)).value()
                commands.append("TL " + ch + " " + str(val))
                
            for cmd in commands:
                self.daq.put(cmd)
                self.logger.info("Set threshold of channel %s to %s" %(cmd.split()[1],cmd.split()[2]))

        self.daq.put('TL')
  
    def config_menu(self):
        """
        Show the config dialog
        """
        gatewidth = 0.
        # get the actual channels...
        self.daq.put('DC')
        # wait explicitely till the channels get loaded
        self.logger.info("loading channel information...")
        time.sleep(1)

        config_window = ConfigDialog(self.channelcheckbox_0,self.channelcheckbox_1,self.channelcheckbox_2,self.channelcheckbox_3,self.coincidencecheckbox_0,self.coincidencecheckbox_1,self.coincidencecheckbox_2,self.coincidencecheckbox_3,self.vetocheckbox,self.vetocheckbox_0,self.vetocheckbox_1,self.vetocheckbox_2)
        rv = config_window.exec_()
        if rv == 1:
            
            chan0_active = config_window.findChild(QtGui.QCheckBox,QtCore.QString("channelcheckbox_0")).isChecked() 
            chan1_active = config_window.findChild(QtGui.QCheckBox,QtCore.QString("channelcheckbox_1")).isChecked() 
            chan2_active = config_window.findChild(QtGui.QCheckBox,QtCore.QString("channelcheckbox_2")).isChecked() 
            chan3_active = config_window.findChild(QtGui.QCheckBox,QtCore.QString("channelcheckbox_3")).isChecked() 
            singles = config_window.findChild(QtGui.QRadioButton,QtCore.QString("coincidencecheckbox_0")).isChecked() 
            if singles:
                self.tabwidget.ratewidget.do_not_show_trigger = True
            else:             
                self.tabwidget.ratewidget.do_not_show_trigger = False
            
            twofold   = config_window.findChild(QtGui.QRadioButton,QtCore.QString("coincidencecheckbox_1")).isChecked() 
            threefold = config_window.findChild(QtGui.QRadioButton,QtCore.QString("coincidencecheckbox_2")).isChecked() 
            fourfold  = config_window.findChild(QtGui.QRadioButton,QtCore.QString("coincidencecheckbox_3")).isChecked() 

            veto    = config_window.findChild(QtGui.QGroupBox,QtCore.QString("vetocheckbox")).isChecked()
            vetochan1 = config_window.findChild(QtGui.QRadioButton,QtCore.QString("vetocheckbox_0")).isChecked()
            vetochan2 = config_window.findChild(QtGui.QRadioButton,QtCore.QString("vetocheckbox_1")).isChecked()
            vetochan3 = config_window.findChild(QtGui.QRadioButton,QtCore.QString("vetocheckbox_2")).isChecked()
            
            tmp_msg = ''
            if veto:
                if vetochan1:
                    tmp_msg += '01'
                elif vetochan2:
                    tmp_msg += '10'
                elif vetochan3:
                    tmp_msg += '11'
                else:
                    tmp_msg += '00' 
            else:
                tmp_msg += '00'
    
            coincidence_set = False
            for coincidence in [(singles,'00'),(twofold,'01'),(threefold,'10'),(fourfold,'11')]:
                if coincidence[0]:
                    tmp_msg += coincidence[1]
                    coincidence_set = True
            
            # else case, just in case
            if not coincidence_set:
                tmp_msg += '00'
    
            # now calculate the correct expression for the first
            # four bits
            self.logger.debug("The first four bits are set to %s" %tmp_msg)
            msg = 'WC 00 ' + hex(int(''.join(tmp_msg),2))[-1].capitalize()
    
            channel_set = False
            enable = ['0','0','0','0']
            for channel in enumerate([chan3_active,chan2_active,chan1_active,chan0_active]):
                if channel[1]:
                    enable[channel[0]] = '1'
                    channel_set = True
            
            if not channel_set:
                msg += '0'
                
            else:
                msg += hex(int(''.join(enable),2))[-1].capitalize()
            
            self.daq.put(msg)
            self.logger.info('The following message was sent to DAQ: %s' %msg)

            self.logger.debug('channel0 selected %s' %chan0_active)
            self.logger.debug('channel1 selected %s' %chan1_active)
            self.logger.debug('channel2 selected %s' %chan2_active)
            self.logger.debug('channel3 selected %s' %chan3_active)
            self.logger.debug('coincidence singles %s' %singles)
            self.logger.debug('coincidence twofold %s' %twofold)
            self.logger.debug('coincidence threefold %s' %threefold)
            self.logger.debug('coincidence fourfold %s' %fourfold)
        self.daq.put('DC')
           
    def advanced_menu(self):
        """
        Show a config dialog for advanced options, ie. gatewidth, interval for the rate measurement, options for writing pulsefile and the nostatus option
        """
        gatewidth = 0.
        # get the actual channels...
        self.daq.put('DC')
        # wait explicitely till the channels get loaded
        self.logger.info("loading channel information...")
        time.sleep(1)

        adavanced_window = AdvancedDialog(self.coincidence_time,self.timewindow,self.nostatus)
        rv = adavanced_window.exec_()
        if rv == 1:
            _timewindow = float(adavanced_window.findChild(QtGui.QDoubleSpinBox,QtCore.QString("timewindow")).value())
            _gatewidth = bin(int(adavanced_window.findChild(QtGui.QSpinBox,QtCore.QString("gatewidth")).value())/10).replace('0b','').zfill(16)
            _nostatus = adavanced_window.findChild(QtGui.QCheckBox,QtCore.QString("nostatus")).isChecked()
            
            _03 = format(int(_gatewidth[0:8],2),'x').zfill(2)
            _02 = format(int(_gatewidth[8:16],2),'x').zfill(2)
            tmp_msg = 'WC 03 '+str(_03)
            self.daq.put(tmp_msg)
            tmp_msg = 'WC 02 '+str(_02)
            self.daq.put(tmp_msg)
            if _timewindow < 0.01 or _timewindow > 10000.:
                      self.logger.warning("Timewindow too small or too big, resetting to 5 s.")
                      self.timewindow = 5.0
            else:
                self.timewindow = _timewindow
            self.widgetupdater.start(self.timewindow*1000)
            self.nostatus = not _nostatus

            self.logger.debug('Writing gatewidth WC 02 %s WC 03 %s' %(_02,_03))
            self.logger.debug('Setting timewindow to %.2f ' %(_timewindow))
            self.logger.debug('Switching nostatus option to %s' %(_nostatus))

        self.daq.put('DC')
             
    def help_menu(self):
        """
        Show a simple help menu
        """
        help_window = HelpDialog()
        help_window.exec_()
        
    def about_menu(self):
        """
        Show a link to the online documentation
        """
        QtGui.QMessageBox.information(self,
                  "about muonic",
                  "version: %s\n source located at: %s"%(__version__,__source_location__))

    def sphinxdoc_menu(self):
        """
        Show the sphinx documentation that comes with muonic in a
        browser
        """
        #docs = (os.path.join(DOCPATH,"index.html"))
        docs = __docs_hosted_at__
        self.logger.debug("Opening docs from %s" %docs)
        success = webbrowser.open(docs)
        if not success:
            self.logger.warning("Can not open webbrowser! Browse to %s to see the docs" %docs)

    def manualdoc_menu(self):
        """
        Show the manual that comes with muonic in a pdf viewer
        """
        docs = (os.path.join(DOCPATH,"manual.pdf"))

        self.logger.info("opening docs from %s" %docs)
        success = webbrowser.open(docs)
        if not success:
            self.logger.warning("Can not open PDF reader!")


                
    def get_thresholds_from_queue(self,msg):
        """
        Explicitely scan message for threshold information
        Return True if found, else False
        """
        if msg.startswith('TL') and len(msg) > 9:
            msg = msg.split('=')
            self.threshold_ch0 = int(msg[1][:-2])
            self.threshold_ch1 = int(msg[2][:-2])
            self.threshold_ch2 = int(msg[3][:-2])
            self.threshold_ch3 = int(msg[4]     )
            self.logger.debug("Got Thresholds %i %i %i %i" %(self.threshold_ch0,self.threshold_ch1,self.threshold_ch2,self.threshold_ch3))
            return True
        else:
            return False
        
    def get_channels_from_queue(self,msg):
        """
        Explicitely scan message for channel information
        Return True if found, else False

        DC gives :
        DC C0=23 C1=71 C2=0A C3=00
        
        Which has the meaning:

        MM - 00 -> 8bits for channel enable/disable, coincidence and veto
        |7   |6   |5          |4          |3       |2       |1 |0       |
        |veto|veto|coincidence|coincidence|channel3|channel2|channel1|channel0|
        ---------------------------bits-------------------------------------
        Set bits for veto:
        ........................
        00 - ch0 is veto
        01 - ch1 is veto
        10 - ch2 is veto
        11 - ch3 is veto
        ........................
        Set bits for coincidence
        ........................
        00 - singles
        01 - twofold
        10 - threefold
        11 - fourfold
        """
        if msg.startswith('DC ') and len(msg) > 25:
            msg = msg.split(' ')
            self.coincidence_time = msg[4].split('=')[1]+ msg[3].split('=')[1]
            msg = bin(int(msg[1][3:], 16))[2:].zfill(8)
            vetoconfig = msg[0:2]
            coincidenceconfig = msg[2:4]
            channelconfig = msg[4:8]

            self.coincidence_time = int(self.coincidence_time, 16)*10
            
            self.vetocheckbox_0 = False
            self.vetocheckbox_1 = False
            self.vetocheckbox_2 = False
            self.vetocheckbox = True

            if str(channelconfig[3]) == '0':
                self.channelcheckbox_0 = False
            else:
                self.channelcheckbox_0 = True

            if str(channelconfig[2]) == '0':
                self.channelcheckbox_1 = False
            else:
                self.channelcheckbox_1 = True

            if str(channelconfig[1]) == '0':
                self.channelcheckbox_2 = False
            else:
                self.channelcheckbox_2 = True
            if str(channelconfig[0]) == '0':
                self.channelcheckbox_3 = False
            else:
                self.channelcheckbox_3 = True
            if str(coincidenceconfig) == '00':
                self.coincidencecheckbox_0 = True
            else:
                self.coincidencecheckbox_0 = False
            if str(coincidenceconfig) == '01':
                self.coincidencecheckbox_1 = True
            else:
                self.coincidencecheckbox_1 = False
            if str(coincidenceconfig) == '10':
                self.coincidencecheckbox_2 = True
            else:
                self.coincidencecheckbox_2 = False

            if str(coincidenceconfig) == '11':
                self.coincidencecheckbox_3 = True
            else:
                self.coincidencecheckbox_3 = False


            if str(vetoconfig) == '00':
                self.vetocheckbox = False
            else:
                if str(vetoconfig) == '01': self.vetocheckbox_0 = True
                if str(vetoconfig) == '10': self.vetocheckbox_1 = True
                if str(vetoconfig) == '11': self.vetocheckbox_2 = True
            
            self.logger.debug('Coincidence timewindow %s ns' %(str(self.coincidence_time)))
            self.logger.debug("Got channel configurations: %i %i %i %i" %(self.channelcheckbox_0,self.channelcheckbox_1,self.channelcheckbox_2,self.channelcheckbox_3))
            self.logger.debug("Got coincidence configurations: %i %i %i %i" %(self.coincidencecheckbox_0,self.coincidencecheckbox_1,self.coincidencecheckbox_2,self.coincidencecheckbox_3))
            self.logger.debug("Got veto configurations: %i %i %i %i" %(self.vetocheckbox,self.vetocheckbox_0,self.vetocheckbox_1,self.vetocheckbox_2))

            return True
        else:
            return False

    # this functions gets everything out of the daq
    # All calculations should happen here
    def processIncoming(self):
        """
        Handle all the messages currently in the daq 
        and pass the result to the corresponding widgets
        """
        while self.daq.data_available():

            try:
                msg = self.daq.get(0)
            except DAQIOError:
                self.logger.debug("Queue empty!")
                return None

            self.daq_msg = msg #make it public for daughters
            # Check contents of message and do what it says
            self.tabwidget.daqwidget.calculate()
            if (self.tabwidget.gpswidget.active and self.tabwidget.gpswidget.isEnabled()):
                if len(self.tabwidget.gpswidget.gps_dump) <= self.tabwidget.gpswidget.read_lines:
                    self.tabwidget.gpswidget.gps_dump.append(msg)
                if len(self.tabwidget.gpswidget.gps_dump) == self.tabwidget.gpswidget.read_lines:
                    self.tabwidget.gpswidget.calculate()
                continue

            if (self.tabwidget.statuswidget.isVisible()) and self.tabwidget.statuswidget.active:
                self.tabwidget.statuswidget.update()

            if msg.startswith('DC') and len(msg) > 2 and self.tabwidget.decaywidget.active:
                try:
                    split_msg = msg.split(" ")
                    self.tabwidget.decaywidget.previous_coinc_time_03 = split_msg[4].split("=")[1]
                    self.tabwidget.decaywidget.previous_coinc_time_02 = split_msg[3].split("=")[1]
                except:
                    self.logger.debug('Wrong DC command.')
                continue

            # check for threshold information
            if self.get_thresholds_from_queue(msg):
                continue

            if self.get_channels_from_queue(msg):
                continue

            # status messages
            if msg.startswith('ST') or len(msg) < 50:
                continue

            if self.tabwidget.ratewidget.calculate():
                continue
            
            if (self.tabwidget.decaywidget.active or\
                        self.tabwidget.pulseanalyzerwidget.active or\
                        self.pulsefilename or\
                        self.tabwidget.velocitywidget.active): #self.showpulses or self.pulsefilename) :
                    self.pulses = self.pulseextractor.extract(msg)

            self.widgetCalculate()


    def widgetCalculate(self):
        """
        Starts the widgets calculate function inside the processIncoming. Set active flag (second parameter in the calculate_widgets list) to True if it should run only when the widget is active.
        """
        for widget in self.tabwidget.pulse_widgets:
            if widget.active and (self.pulses is not None):
                widget.calculate()

    def widgetUpdate(self):
        """
        Update the widgets
        """
        for widg in self.tabwidget.dynamic_widgets:
            if widg.active:
                widg.update()

    def closeEvent(self, ev):
        """
        Is triggered when the window is closed, we have to reimplement it
        to provide our special needs for the case the program is ended.
        """

        self.logger.info('Attempting to close Window!')
        # ask kindly if the user is really sure if she/he wants to exit
        reply = QtGui.QMessageBox.question(self, 'Attention!',
                'Do you really want to exit?', QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)

        if reply == QtGui.QMessageBox.Yes:
            self.timer.stop()
            self.widgetupdater.stop()
            now = datetime.datetime.now()

            # close the RAW file (if any)
            if self.tabwidget.daqwidget.write_file:
                self.tabwidget.daqwidget.write_file = False
                mtime = now - self.raw_mes_start
                mtime = round(mtime.seconds/(3600.),2) + mtime.days*86400
                self.logger.info("The raw data was written for %f hours" % mtime)
                newrawfilename = self.rawfilename.replace("HOURS",str(mtime))
                shutil.move(self.rawfilename,newrawfilename)
                self.tabwidget.daqwidget.outputfile.close()

            if self.tabwidget.decaywidget.active:
                mtime = now - self.tabwidget.decaywidget.dec_mes_start
                mtime = round(mtime.seconds/(3600.),2) + mtime.days*86400
                self.logger.info("The muon decay measurement was active for %f hours" % mtime)
                newmufilename = self.decayfilename.replace("HOURS",str(mtime))
                shutil.move(self.decayfilename,newmufilename)

            if self.pulsefilename:
                old_pulsefilename = self.pulsefilename
                # no pulses shall be extracted any more, 
                # this means changing lots of switches
                self.pulsefilename = False
                self.showpulses = False
                self.pulseextractor.close_file()
                mtime = now - self.pulse_mes_start
                mtime = round(mtime.seconds/(3600.),2) + mtime.days*86400
                self.logger.info("The pulse extraction measurement was active for %f hours" % mtime)
                newpulsefilename = old_pulsefilename.replace("HOURS",str(mtime))
                shutil.move(old_pulsefilename,newpulsefilename)
              
            self.tabwidget.ratewidget.data_file_write = False
            self.tabwidget.ratewidget.data_file.close()
            mtime = now - self.tabwidget.ratewidget.rate_mes_start
            #print 'HOURS ', now, '|', mtime, '|', mtime.days, '|', str(mtime)                
            mtime = round(mtime.seconds/(3600.),2) + mtime.days*86400
            #print 'new mtime ', mtime, str(mtime)
            self.logger.info("The rate measurement was active for %f hours" % mtime)
            newratefilename = self.filename.replace("HOURS",str(mtime))
            #print 'new raw name', newratefilename
            shutil.move(self.filename,newratefilename)
            time.sleep(0.5)
            self.tabwidget.writefile = False
            try:
                self.tabwidget.decaywidget.mu_file.close()
 
            except AttributeError:
                pass

            self.emit(QtCore.SIGNAL('lastWindowClosed()'))
            self.close()

        else: # don't close the mainwindow
            ev.ignore()




import multiprocessing as mult
import logging
import re
import Queue

try:
    import zmq
except ImportError:
    print "no zmq installed..."

from SimDaqConnection import SimDaqConnection,SimDaqServer
from DaqConnection import DaqConnection,DaqServer

class DAQIOError(IOError):
    pass


class DAQProvider:
    """
    Launch the main part of the GUI and the worker threads. periodicCall and
    endApplication could reside in the GUI part, but putting them here
    means that you have all the thread controls in a single place.
    """

    def __init__(self,logger=None,sim=False):

        self.outqueue = mult.Queue()
        self.inqueue  = mult.Queue()
            
        self.running = 1
        self.good_pattern = re.compile("^[a-zA-Z0-9+-.,:()=$/#?!%_@*|~' ]*[\n\r]*$")
        # get option parser options
        if logger is None:
            logger = logging.getLogger()
        self.logger = logger
        self.sim = sim

        if self.sim:
            self.daq = SimDaqConnection(self.inqueue, self.outqueue, self.logger)

        else:
            self.daq = DaqConnection(self.inqueue, self.outqueue, self.logger)
        
        # Set up the thread to do asynchronous I/O
        # More can be made if necessary
        # Set daemon flag so that the threads finish when the main app finishes
        self.readthread = mult.Process(target=self.daq.read,name="pREADER")
        self.readthread.daemon = True
        self.readthread.start()
        if not self.sim:
            self.writethread = mult.Process(target=self.daq.write,name="pWRITER")
            self.writethread.daemon = True
            self.writethread.start()
        
    def get(self,*args):
        """
        Get something from the daq
        """

        try:
            line = self.outqueue.get(*args)
        except Queue.Empty:
            raise DAQIOError("Queue is empty")
        
        if self.good_pattern.match(line) is None:
            # Do something more sensible here,     like stopping the DAQ
            # then wait until service is restar    ted?
            self.logger.warning("Got garbage from the DAQ: %s"%line.rstrip('\r\n'))
            return None
        
        return line
        

    def put(self,*args):
        """
        Send information to the daq
        """
        self.inqueue.put(*args)

    def data_available(self):
        """
        is new data from daq available
        """
        size = None
        try:
            size = self.outqueue.qsize()
        except NotImplementedError:
            self.logger.debug("Running Mac version of muonic.")
            size = not self.outqueue.empty()
        return size

class DAQClient(DAQProvider):
    
    
    def __init__(self,port,logger=None,root=None):
           
        self.running = 1
        self.root = root
        self.good_pattern = re.compile("^[a-zA-Z0-9+-.,:()=$/#?!%_@*|~' ]*[\n\r]*$")
        # get option parser options
        if logger is None:
            logger = logging.getLogger()
        self.logger = logger
        self.setup_socket(port)

     
    def setup_socket(self,port):
        #port = "5556"
        context = zmq.Context()
        self.socket = context.socket(zmq.PAIR)
        self.socket.connect("tcp://127.0.0.1:%s" % port)
        self.socket_port = port 
        
    def get(self,*args):
        """
        Get something from the daq
        """
        
        try:
            line = self.socket.recv_string()
        except Queue.Empty:
            raise DAQIOError("Queue is empty")
        
        if self.good_pattern.match(line) is None:
            # Do something more sensible here,     like stopping the DAQ
            # then wait until service is restar    ted?
            self.logger.warning("Got garbage from the DAQ: %s"%line.rstrip('\r\n'))
            return None
            #raise DAQIOError("Queue contains garbage!")
        
        return line
        

    def put(self,*args):
        """
        Send information to the daq
        """
        self.socket.send_string(*args)

    def data_available(self):
        """
        is new data from daq available
        """

        return self.socket.poll(200)



#! /usr/bin/env python

"""
Reads a given text and returns just the numbers from it
"""

import sys


def get_numbers():

   for line in open(sys.argv[1]).readlines():
        fields = line.split(' ')
        for field in fields:
            try:
                print float(field)
            except ValueError:
                pass

if __name__ == "__main__":
    get_numbers()


# vim: ai ts=4 sts=4 et sw=4
